unit Unit1;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes,
  System.Variants,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  FMX.Objects, FMX.ListBox, FMX.Layouts, FMX.Media, FMX.Filter,
  FMX.Filter.Effects;

type
  TForm1 = class(TForm)
    Layout1: TLayout;
    StartButton: TButton;
    ComboBox1: TComboBox;
    Image1: TImage;
    procedure FormCreate(Sender: TObject);
    procedure ComboBox1Change(Sender: TObject);
    procedure StartButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { private 錾 }
  public
    { public 錾 }
    VideoCamera: TVideoCaptureDevice;
    Effect: TImageFXEffect;
    procedure SampleBufferSync;
    procedure SampleBufferReady(Sender: TObject; const ATime: TMediaTime);
  end;

var
  Form1: TForm1;

implementation

{$R *.fmx}

/// <summary>
/// TCYύX摜擾
/// </summary>
/// <param name="Bitmap">
/// 摜
/// </param>
/// <param name="Height">
/// V摜̍
/// </param>
/// <param name="Width">
/// V摜̕
/// </param>
function ResizeImage(Bitmap: TBitmap; Height, Width: Integer): TBitmap;
var
  SrcRect: TRectF;
  DstRect: TRectF;
begin
  SrcRect.Left := 0;
  SrcRect.Top := 0;
  SrcRect.Bottom := Bitmap.Height;
  SrcRect.Right := Bitmap.Width;

  DstRect.Left := 0;
  DstRect.Top := 0;
  DstRect.Bottom := Height;
  DstRect.Right := Width;

  Result := TBitmap.Create;
  Result.Height := Height;
  Result.Width := Width;
  Result.Canvas.DrawBitmap(Bitmap, SrcRect, DstRect, 1.0);
end;

{ TForm1 }

procedure TForm1.ComboBox1Change(Sender: TObject);
begin
  VideoCamera := TVideoCaptureDevice
    (TCaptureDeviceManager.Current.GetDevicesByName(ComboBox1.Selected.Text));
  if (VideoCamera <> nil) then
  begin
    StartButton.Enabled := true;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  DeviceList: TCaptureDeviceList;
  i: integer;
begin
  DeviceList := TCaptureDeviceManager.Current.GetDevicesByMediaType
    (TMediaType.Video);
  for i := 0 to DeviceList.Count - 1 do
  begin
    ComboBox1.Items.Add(DeviceList[i].Name);
  end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  if VideoCamera <> nil then
    VideoCamera.StopCapture;
end;

procedure TForm1.SampleBufferReady(Sender: TObject; const ATime: TMediaTime);
begin
  TThread.Synchronize(TThread.CurrentThread, SampleBufferSync);
end;

procedure TForm1.SampleBufferSync;
var
  Bitmap1, Bitmap2: TBitmap;
begin
  Bitmap1:= TBitmap.Create;
  VideoCamera.SampleBufferToBitmap(Bitmap1, true);
  Bitmap2 := ResizeImage(Bitmap1, Trunc(Image1.Height), Trunc(Image1.Width));
  Image1.Bitmap.Assign(Bitmap2);

  Bitmap1.Free;
  Bitmap2.Free;
end;

procedure TForm1.StartButtonClick(Sender: TObject);
begin
  if (VideoCamera <> nil) then
  begin
    if (VideoCamera.State = TCaptureDeviceState.Stopped) then
    begin
      VideoCamera.OnSampleBufferReady := SampleBufferReady;
      VideoCamera.StartCapture;
      StartButton.Text := 'Stop';
    end
    else
    begin
      VideoCamera.StopCapture;
      StartButton.Text := 'Start';
    end;
  end
  else
  begin
    Caption := 'Video capture devices not available.';
  end;
end;

initialization
  GlobalUseDirect2D := False;
//  GlobalUseDirect2D := True;
end.
